package controllers

import (
	"encoding/json"

	oclib "cloud.o-forge.io/core/oc-lib"
	beego "github.com/beego/beego/v2/server/web"
)

// Operations about data
type DataController struct {
	beego.Controller
}

var data_collection = oclib.LibDataEnum(oclib.DATA_RESOURCE)

// @Title Update
// @Description create datas
// @Param	id		path 	string	true		"the data id you want to get"
// @Param	body		body 	models.data	true		"The data content"
// @Success 200 {data} models.data
// @router /:id [put]
func (o *DataController) Put() {
	// store and return Id or post with UUID
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	var res map[string]interface{}
	id := o.Ctx.Input.Param(":id")
	json.Unmarshal(o.Ctx.Input.CopyBody(10000), &res)
	o.Data["json"] = oclib.NewRequest(data_collection, user, peerID, groups, nil).UpdateOne(res, id)
	o.ServeJSON()
}

// @Title Create
// @Description create data
// @Param	data		body 	json	true		"body for data content (Json format)"
// @Success 200 {data} models.data
// @router / [post]
func (o *DataController) Post() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	var res map[string]interface{}
	json.Unmarshal(o.Ctx.Input.CopyBody(10000), &res)
	o.Data["json"] = oclib.NewRequest(data_collection, user, peerID, groups, nil).StoreOne(res)
	o.ServeJSON()
}

// @Title GetAll
// @Description find data by id
// @Param	is_draft		query 	string	false		"draft wished"
// @Success 200 {data} models.data
// @router / [get]
func (o *DataController) GetAll() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	isDraft := o.Ctx.Input.Query("is_draft")
	o.Data["json"] = oclib.NewRequest(data_collection, user, peerID, groups, nil).LoadAll(isDraft == "true")
	o.ServeJSON()
}

// @Title Get
// @Description find data by key word
// @Param	search		path 	string	true		"the search you want to get"
// @Param	is_draft		query 	string	false		"draft wished"

// @Success 200 {data} models.data
// @router /search/:search [get]
func (o *DataController) Search() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	search := o.Ctx.Input.Param(":search")
	isDraft := o.Ctx.Input.Query("is_draft")
	o.Data["json"] = oclib.NewRequest(data_collection, user, peerID, groups, nil).Search(nil, search, isDraft == "true")
	o.ServeJSON()
}

// @Title Get
// @Description find data by id
// @Param	id		path 	string	true		"the id you want to get"
// @Success 200 {data} models.data
// @router /:id [get]
func (o *DataController) Get() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	id := o.Ctx.Input.Param(":id")
	o.Data["json"] = oclib.NewRequest(data_collection, user, peerID, groups, nil).LoadOne(id)
	o.ServeJSON()
}

// @Title Delete
// @Description delete the data
// @Param	id		path 	string	true		"The id you want to delete"
// @Success 200 {data} delete success!
// @router /:id [delete]
func (o *DataController) Delete() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	id := o.Ctx.Input.Param(":id")
	o.Data["json"] = oclib.NewRequest(data_collection, user, peerID, groups, nil).DeleteOne(id)
	o.ServeJSON()
}
