package controllers

import (
	"encoding/json"

	oclib "cloud.o-forge.io/core/oc-lib"
	beego "github.com/beego/beego/v2/server/web"
)

// Operations about processing
type ProcessingController struct {
	beego.Controller
}

var processing_collection = oclib.LibDataEnum(oclib.PROCESSING_RESOURCE)

// @Title Update
// @Description create processings
// @Param	id		path 	string	true		"the processing id you want to get"
// @Param	body		body 	models.processing	true		"The processing content"
// @Success 200 {processing} models.processing
// @router /:id [put]
func (o *ProcessingController) Put() {
	// store and return Id or post with UUID
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	var res map[string]interface{}
	id := o.Ctx.Input.Param(":id")
	json.Unmarshal(o.Ctx.Input.CopyBody(10000), &res)
	o.Data["json"] = oclib.NewRequest(processing_collection, user, peerID, groups, nil).UpdateOne(res, id)
	o.ServeJSON()
}

// @Title Create
// @Description create processing
// @Param	processing		body 	json	true		"body for processing content (Json format)"
// @Success 200 {processing} models.processing
// @router / [post]
func (o *ProcessingController) Post() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	var res map[string]interface{}
	json.Unmarshal(o.Ctx.Input.CopyBody(10000), &res)
	o.Data["json"] = oclib.NewRequest(processing_collection, user, peerID, groups, nil).StoreOne(res)
	o.ServeJSON()
}

// @Title GetAll
// @Description find processing by id
// @Param	is_draft		query 	string	false		"draft wished"
// @Success 200 {processing} models.processing
// @router / [get]
func (o *ProcessingController) GetAll() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	isDraft := o.Ctx.Input.Query("is_draft")
	o.Data["json"] = oclib.NewRequest(processing_collection, user, peerID, groups, nil).LoadAll(isDraft == "true")
	o.ServeJSON()
}

// @Title Get
// @Description find processing by key word
// @Param	search		path 	string	true		"the search you want to get"
// @Param	is_draft		query 	string	false		"draft wished"
// @Success 200 {processing} models.processing
// @router /search/:search [get]
func (o *ProcessingController) Search() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	search := o.Ctx.Input.Param(":search")
	isDraft := o.Ctx.Input.Query("is_draft")
	o.Data["json"] = oclib.NewRequest(processing_collection, user, peerID, groups, nil).Search(nil, search, isDraft == "true")
	o.ServeJSON()
}

// @Title Get
// @Description find processing by id
// @Param	id		path 	string	true		"the id you want to get"
// @Success 200 {processing} models.processing
// @router /:id [get]
func (o *ProcessingController) Get() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	id := o.Ctx.Input.Param(":id")
	o.Data["json"] = oclib.NewRequest(processing_collection, user, peerID, groups, nil).LoadOne(id)
	o.ServeJSON()
}

// @Title Delete
// @Description delete the processing
// @Param	id		path 	string	true		"The id you want to delete"
// @Success 200 {processing} delete success!
// @router /:id [delete]
func (o *ProcessingController) Delete() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	id := o.Ctx.Input.Param(":id")
	o.Data["json"] = oclib.NewRequest(processing_collection, user, peerID, groups, nil).DeleteOne(id)
	o.ServeJSON()
}
