package controllers

import (
	"encoding/json"

	oclib "cloud.o-forge.io/core/oc-lib"
	beego "github.com/beego/beego/v2/server/web"
)

// Operations about workflow
type WorkflowController struct {
	beego.Controller
}

var workflow_collection = oclib.LibDataEnum(oclib.WORKFLOW_RESOURCE)

// @Title Update
// @Description create workflows
// @Param	id		path 	string	true		"the workflow id you want to get"
// @Param	body		body 	models.workflow	true		"The workflow content"
// @Success 200 {workflow} models.workflow
// @router /:id [put]
func (o *WorkflowController) Put() {
	// store and return Id or post with UUID
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	var res map[string]interface{}
	id := o.Ctx.Input.Param(":id")
	json.Unmarshal(o.Ctx.Input.CopyBody(10000), &res)
	o.Data["json"] = oclib.NewRequest(workflow_collection, user, peerID, groups, nil).UpdateOne(res, id)
	o.ServeJSON()
}

// @Title Create
// @Description create workflow
// @Param	workflow		body 	json	true		"body for workflow content (Json format)"
// @Success 200 {workflow} models.workflow
// @router / [post]
func (o *WorkflowController) Post() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	var res map[string]interface{}
	json.Unmarshal(o.Ctx.Input.CopyBody(10000), &res)
	o.Data["json"] = oclib.NewRequest(workflow_collection, user, peerID, groups, nil).StoreOne(res)
	o.ServeJSON()
}

// @Title GetAll
// @Description find workflow by id
// @Param	is_draft		query 	string	false		"draft wished"
// @Success 200 {workflow} models.workflow
// @router / [get]
func (o *WorkflowController) GetAll() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	isDraft := o.Ctx.Input.Query("is_draft")
	o.Data["json"] = oclib.NewRequest(workflow_collection, user, peerID, groups, nil).LoadAll(isDraft == "true")
	o.ServeJSON()
}

// @Title Get
// @Description find workflow by key word
// @Param	search		path 	string	true		"the search you want to get"
// @Param	is_draft		query 	string	false		"draft wished"
// @Success 200 {workflow} models.workflow
// @router /search/:search [get]
func (o *WorkflowController) Search() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	search := o.Ctx.Input.Param(":search")
	isDraft := o.Ctx.Input.Query("is_draft")
	o.Data["json"] = oclib.NewRequest(workflow_collection, user, peerID, groups, nil).Search(nil, search, isDraft == "true")
	o.ServeJSON()
}

// @Title Get
// @Description find workflow by id
// @Param	id		path 	string	true		"the id you want to get"
// @Success 200 {workflow} models.workflow
// @router /:id [get]
func (o *WorkflowController) Get() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	id := o.Ctx.Input.Param(":id")
	o.Data["json"] = oclib.NewRequest(workflow_collection, user, peerID, groups, nil).LoadOne(id)
	o.ServeJSON()
}

// @Title Delete
// @Description delete the workflow
// @Param	id		path 	string	true		"The id you want to delete"
// @Success 200 {workflow} delete success!
// @router /:id [delete]
func (o *WorkflowController) Delete() {
	user, peerID, groups := oclib.ExtractTokenInfo(*o.Ctx.Request)
	id := o.Ctx.Input.Param(":id")
	o.Data["json"] = oclib.NewRequest(workflow_collection, user, peerID, groups, nil).DeleteOne(id)
	o.ServeJSON()
}
