package controllers

import (
	"cloud.o-forge.io/core/oc-lib/models/common/enum"
	"cloud.o-forge.io/core/oc-lib/models/common/pricing"
	"cloud.o-forge.io/core/oc-lib/models/resources"
	"cloud.o-forge.io/core/oc-lib/tools"
	beego "github.com/beego/beego/v2/server/web"
)

// Operations about resource
type EnumController struct {
	beego.Controller
}

// @Title EnumStorageType
// @Description get list of StorageType
// @Success 200 {resource} models.resource
// @router /storage/type [get]
func (o *EnumController) EnumStorageType() {
	enumMap := map[int]string{}
	for _, d := range enum.TypeList() {
		enumMap[int(d)] = d.String()
	}
	o.Data["json"] = map[string]interface{}{"data": enumMap, "code": 200, "error": ""}
	o.ServeJSON()
}

// @Title EnumStorageSize
// @Description get list of StorageSize
// @Success 200 {resource} models.resource
// @router /storage/size [get]
func (o *EnumController) EnumStorageSize() {
	enumMap := map[int]string{}
	for _, d := range enum.SizeList() {
		enumMap[int(d)] = d.String()
	}
	o.Data["json"] = map[string]interface{}{"data": enumMap, "code": 200, "error": ""}
	o.ServeJSON()
}

// @Title EnumInfrastructure
// @Description get list of Infrastructure
// @Success 200 {resource} models.resource
// @router /infrastructure [get]
func (o *EnumController) EnumInfrastructure() {
	enumMap := map[int]string{}
	for _, d := range enum.InfrastructureList() {
		enumMap[int(d)] = d.String()
	}
	o.Data["json"] = map[string]interface{}{"data": enumMap, "code": 200, "error": ""}
	o.ServeJSON()
}

// @Title BookingStatus
// @Description get list of Infrastructure
// @Success 200 {resource} models.resource
// @router /booking/status [get]
func (o *EnumController) EnumBookingStatus() {
	enumMap := map[int]string{}
	for _, d := range enum.StatusList() {
		enumMap[int(d)] = d.String()
	}
	o.Data["json"] = map[string]interface{}{"data": enumMap, "code": 200, "error": ""}
	o.ServeJSON()
}

// @Title EnumStatus
// @Description get list of status
// @Success 200 {resource} models.resource
// @router /status [get]
func (o *EnumController) EnumStatus() {
	enumMap := map[int]string{}
	for _, d := range enum.StatusList() {
		enumMap[int(d)] = d.String()
	}
	o.Data["json"] = map[string]interface{}{"data": enumMap, "code": 200, "error": ""}
	o.ServeJSON()
}

// @Title EnumStrategyBuy
// @Description get list of status
// @Success 200 {resource} models.resource
// @router /pricing/strategy/buy [get]
func (o *EnumController) EnumStrategyBuy() {
	enumMap := map[int]string{}
	for _, d := range pricing.BuyingStrategyList() {
		enumMap[int(d)] = d.String()
	}
	o.Data["json"] = map[string]interface{}{"data": enumMap, "code": 200, "error": ""}
	o.ServeJSON()
}

// @Title EnumStrategyTime
// @Description get list of status
// @Success 200 {resource} models.resource
// @router /pricing/strategy/time [get]
func (o *EnumController) EnumStrategyTime() {
	enumMap := map[int]string{}
	for _, d := range pricing.TimePricingStrategyList() {
		enumMap[int(d)] = d.String()
	}
	o.Data["json"] = map[string]interface{}{"data": enumMap, "code": 200, "error": ""}
	o.ServeJSON()
}

// @Title EnumStrategyPrivilege
// @Description get list of status
// @Success 200 {resource} models.resource
// @router /pricing/strategy/privilege [get]
func (o *EnumController) EnumStrategyPrivilege() {
	enumMap := map[int]string{}
	for _, d := range pricing.ExploitPrivilegeStrategyList() {
		enumMap[int(d)] = d.String()
	}
	o.Data["json"] = map[string]interface{}{"data": enumMap, "code": 200, "error": ""}
	o.ServeJSON()
}

// @Title EnumRefundType
// @Description get list of status
// @Success 200 {resource} models.resource
// @router /pricing/refund/type [get]
func (o *EnumController) EnumRefundType() {
	enumMap := map[int]string{}
	for _, d := range pricing.RefundTypeList() {
		enumMap[int(d)] = d.String()
	}
	o.Data["json"] = map[string]interface{}{"data": enumMap, "code": 200, "error": ""}
	o.ServeJSON()
}

// @Title EnumStrategyPrivilegeStorage
// @Description get list of status
// @Success 200 {resource} models.resource
// @router /pricing/strategy/privilege/storage [get]
func (o *EnumController) EnumStrategyPrivilegeStorage() {
	enumMap := map[int]string{}
	for _, d := range resources.PrivilegeStoragePricingStrategyList() {
		enumMap[int(d)] = d.String()
	}
	o.Data["json"] = map[string]interface{}{"data": enumMap, "code": 200, "error": ""}
	o.ServeJSON()
}

// @Title EnumStrategyStorage
// @Description get list of status
// @Success 200 {resource} models.resource
// @router /pricing/strategy/storage [get]
func (o *EnumController) EnumStrategyStorage() {
	enumMap := map[int]string{}
	for _, d := range resources.StorageResourcePricingStrategyList() {
		enumMap[int(d)] = d.String()
	}
	o.Data["json"] = map[string]interface{}{"data": enumMap, "code": 200, "error": ""}
	o.ServeJSON()
}

// @Title EnumStrategyData
// @Description get list of status
// @Success 200 {resource} models.resource
// @router /pricing/strategy/data [get]
func (o *EnumController) EnumStrategyData() {
	enumMap := map[int]string{}
	for _, d := range resources.DataResourcePricingStrategyList() {
		enumMap[int(d)] = d.String()
	}
	o.Data["json"] = map[string]interface{}{"data": enumMap, "code": 200, "error": ""}
	o.ServeJSON()
}

// @Title EnumResourceType
// @Description get list of status
// @Success 200 {resource} models.resource
// @router /resource/type [get]
func (o *EnumController) EnumResourceType() {
	enumMap := map[int]string{}
	for _, d := range tools.DataTypeList() {
		enumMap[int(d)] = d.String()
	}
	o.Data["json"] = map[string]interface{}{"data": enumMap, "code": 200, "error": ""}
	o.ServeJSON()
}
